require "spectest/version"
require "spectest/logger"
require "spectest/matchers"
require "spectest/platforms"
require "spectest/generators"
require "spectest/enclosers"

module SpecTest
  include Matchers
  include Platforms

  # @return [TerminusSpec::Platforms::WatirWebDriver::PlatformObject] the platform object
  # @return [TerminusSpec::Platforms::SeleniumWebDriver::PlatformObject] the platform object
  attr_reader :platform

  def self.version_info
    "SpecTest version #{SpecTest::VERSION}"
  end

  # This makes sure that any page classes that include SpecTest will be
  # given access to the generator and encloser methods based on web
  # objects that are declared in the page class.
  def self.included(caller)
    caller.extend SpecTest::Generators
    caller.extend SpecTest::Enclosers
  end

  def initialize(browser, visit=nil)
    @browser = browser
    establish_platform_driver_for browser
    goto if visit && respond_to?(:goto)
  end

  # Navigate to the specified URL. The URL can be specified as a domain
  # address or as a file-based link.
  # @param [String] url the full URL to navigate to
  def navigate_to(url)
    # Here is exactly why a platform object is needed. Which version do I call:
    #   @browser.goto(url)  <-- this is for Watir
    #   @browser.navigate.to(url)  <-- this is for Selenium
    # I have to call the platform that has been established. The platform will
    # be responsible for containing the correct call.
    @platform.navigate_to url
  end

  # Returns the title of the current page as displayed in the browser.
  def title
    @platform.title
  end

  # Returns the text of the current page.
  def text
    @platform.text
  end

  # Used to identify a web object as existing within an enclosing object
  # like a frame or an iframe. There is a duplicate method in enclosers.rb.
  # This method needs to be here so that test steps can be placed in an
  # enclosed context.
  #
  # @param [String] identifier how an encloser will be referred to
  # @param encloser a parent encloser that is passed from a previous call
  # @param [optional] block that contains calls to web objects within the encloser
  def is_enclosed_by(identifier, encloser=nil, &block)
    encloser = [] if encloser.nil?
    encloser << identifier
    block.call(encloser)
  end

  private

  def establish_platform_driver_for(browser)
    @platform = platform_for browser, SpecTest::Platforms.list
  end

  def process_block(&block)
    block.arity == 1 ? block.call(self) : self.instance_eval(&block)
  end
end
