module SpecTest
  module Platforms
    module SeleniumWebDriver
      class PlatformObject

        def initialize(browser)
          @browser = browser
        end

        # Platform method to navigate to a specified URL.
        # See SpecTest#navigate_to
        def navigate_to(url)
          @browser.navigate.to url
        end

        # Platform method to retrieve the title for the current page.
        # See SpecTest#title
        def title
          @browser.title
        end

        # Platform method to retrieve text from the current page.
        # See SpecTest#text
        def text
          @browser.find_element(:tag_name, 'body').text
        end

        # Platform method to return a link object.
        # Link objects are of type: SpecTest::WebObjects::Link
        # See SpecTest::Generators#link
        def get_link_for(locator)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::Link, 'a')
          switch_to_encloser(enclosers)
          web_object = @browser.find_element(key, value)
          @browser.switch_to.default_content unless enclosers.nil?
          WebObjects::Link.new(web_object, :platform => :selenium_webdriver)
        end

        # Platform method to click a link object.
        # See SpecTest::Generators#link
        def click_link_for(locator)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::Link, 'a')
          switch_to_encloser(enclosers)
          @browser.find_element(key, value).click
          @browser.switch_to.default_content unless enclosers.nil?
        end

        # Platform method to return a text field object.
        # Text field objects are of type: SpecTest::WebObjects::TextField
        # See SpecTest::Generators#text_field
        def get_text_field_for(locator)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::TextField, 'input', :type => 'text')
          switch_to_encloser(enclosers)
          web_object = @browser.find_element(key, value)
          @browser.switch_to.default_content unless enclosers.nil?
          WebObjects::TextField.new(web_object, :platform => :selenium_webdriver)
        end

        # Platform method to get the value in a text field object.
        # See SpecTest::Generators#text_field
        def get_text_field_value_for(locator)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::TextField, 'input', :type => 'text')
          switch_to_encloser(enclosers)
          value = @browser.find_element(key, value).attribute('value')
          @browser.switch_to.default_content unless enclosers.nil?
          value
        end

        # Platform method to set a value in a text field object.
        # See SpecTest::Generators#text_field
        def set_text_field_value_for(locator, text)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::TextField, 'input', :type => 'text')
          switch_to_encloser(enclosers)
          @browser.find_element(key, value).clear
          @browser.find_element(key, value).send_keys(text)
          @browser.switch_to.default_content unless enclosers.nil?
        end

        # Platform method to return a button object.
        # Button objects are of type: SpecTest::WebObjects::Button
        # See SpecTest::Generators#button
        def get_button_for(locator)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::Button, 'input', :type => 'submit')
          switch_to_encloser(enclosers)
          web_object = @browser.find_element(key, value)
          @browser.switch_to.default_content unless enclosers.nil?
          WebObjects::Button.new(web_object, :platform => :selenium_webdriver)
        end

        # Platform method to click a button object.
        # See SpecTest::Generators#button
        def click_button_for(locator)
          key, value, enclosers = get_platform_locator_for(locator, WebObjects::Button, 'input', :type => 'submit')
          switch_to_encloser(enclosers)
          @browser.find_element(key, value).click
          @browser.switch_to.default_content unless enclosers.nil?
        end
        
        protected

        def get_platform_locator_for(locator, web_object, tag=nil, qualifier=nil)
          enclosers = locator.delete(:frame)
          locator = qualify_with_tagname_for locator, tag, qualifier if tag
          key, value = web_object.have_selenium_find_object_with locator
          return key, value, enclosers
        end

        def qualify_with_tagname_for(locator, tag, qualifier=nil)
          return locator if locator.length < 2 and support_is_provided_for locator, tag, qualifier
          locator[:tag_name] = tag
          if qualifier
            qualifier.each do |key, value|
              locator[key] = value
            end
          end
          locator
        end

        def support_is_provided_for(locator, tag, qualifier)
          return false if locator[:index]
          return false if locator[:text] and tag == 'input' and qualifier[:type] == 'hidden'
          return false if locator[:href] and tag == 'a'
          return false if locator[:text] and ['div', 'span', 'td'].include? tag
          return false if locator[:value] and tag == 'input' and qualifier[:type] == 'submit'
          return false if locator[:value] and tag == 'input' and qualifier[:type] == 'radio'
          true
        end

        def switch_to_encloser(enclosers)
          unless enclosers.nil?
            enclosers.each do |encloser_locator|
              value = encloser_locator.values.first
              @browser.switch_to.frame(value)
            end
          end
        end
        
      end # class PlatformObject
    end # module SeleniumWebDriver
  end # module Platforms
end # module SpecTest

Dir["#{File.dirname(__FILE__)}/../web_objects/**/*.rb"].each { |file| require file }