module SpecTest
  module Platforms
    module WatirWebDriver
      class PlatformObject

        attr_reader :browser

        def initialize(browser)
          @browser = browser
        end

        # Platform method to navigate to a specified URL.
        # See SpecTest#navigate_to
        def navigate_to(url)
          @browser.goto url
        end

        # Platform method to retrieve the title for the current page.
        # See SpecTest#title
        def title
          @browser.title
        end

        # Platform method to retrieve text from the current page.
        # See SpecTest#text
        def text
          @browser.text
        end

        # Platform method to return a link object.
        # Link objects are of type: SpecTest::WebObjects::Link
        # See SpecTest::Generators#link
        def get_link_for(locator)
          locator, enclosers = get_platform_locator_for(locator, WebObjects::Link)
          web_object = @browser.instance_eval "#{nested_by(enclosers)}link(locator)"
          switch_to_default(enclosers)
          WebObjects::Link.new(web_object, :platform => :watir_webdriver)
        end

        # Platform method to click a link object.
        # See SpecTest::Generators#link
        def click_link_for(locator)
          locator, enclosers = get_platform_locator_for(locator, WebObjects::Link)
          @browser.instance_eval "#{nested_by(enclosers)}link(locator).click if locator"
          switch_to_default(enclosers)
        end

        # Platform method to return a text field object.
        # Text field objects are of type: SpecTest::WebObjects::TextField
        # See SpecTest::Generators#text_field
        def get_text_field_for(locator)
          identifier, enclosers = get_platform_locator_for(locator, WebObjects::TextField)
          web_object = @browser.instance_eval "#{nested_by(enclosers)}text_field(locator)"
          switch_to_default(enclosers)
          WebObjects::TextField.new(web_object, :platform => :watir_webdriver)
        end

        # Platform method to get the value in a text field object.
        # See SpecTest::Generators#text_field
        def get_text_field_value_for(locator)
          identifier, enclosers = get_platform_locator_for(locator, WebObjects::TextField)
          value = @browser.instance_eval "#{nested_by(enclosers)}text_field(locator).value"
          switch_to_default(enclosers)
          value
        end

        # Platform method to set a value in a text field object.
        # See SpecTest::Generators#text_field
        def set_text_field_value_for(locator, text)
          identifier, enclosers = get_platform_locator_for(locator, WebObjects::TextField)
          SpecTest::trace("set_text_field_value_for: #{identifier}, #{enclosers}")
          @browser.instance_eval "#{nested_by(enclosers)}text_field(locator).set(text)"
          switch_to_default(enclosers)
        end

        # Platform method to return a button object.
        # Button objects are of type: SpecTest::WebObjects::Button
        # See SpecTest::Generators#button
        def get_button_for(locator)
          identifier, enclosers = get_platform_locator_for(locator, WebObjects::Button)
          web_object = @browser.instance_eval "#{nested_by(enclosers)}button(locator)"
          switch_to_default(enclosers)
          WebObjects::Button.new(web_object, :platform => :watir_webdriver)
        end

        # Platform method to click a button object.
        # See SpecTest::Generators#button
        def click_button_for(locator)
          identifier, enclosers = get_platform_locator_for(locator, WebObjects::Button)
          @browser.instance_eval "#{nested_by(enclosers)}button(locator).click"
          switch_to_default(enclosers)
        end

        protected

        def get_platform_locator_for(locator, web_object, tag=nil)
          enclosers = locator.delete(:frame)
          SpecTest::trace("get_platform_locator_for (enclosers): #{enclosers}")
          locator = qualify_with_tagname_for locator, tag if tag
          locator = web_object.have_watir_find_object_with locator
          return locator, enclosers
        end
        
        def qualify_with_tagname_for(locator, tag)
          return locator if locator.length < 2 and not locator[:name]
          locator[:tag_name] = tag if locator[:name]
          locator
        end

        def nested_by(enclosers)
          return if enclosers.nil?

          encloser_locator = ''

          enclosers.each do |id|
            value = id.values.first
            encloser_locator += "frame(:#{id.keys.first} => #{value})." if value.to_s.is_integer?
            encloser_locator += "frame(:#{id.keys.first} => '#{value}')." unless value.to_s.is_integer?
          end
          encloser_locator
        end

        def switch_to_default(enclosers)
          @browser.wd.switch_to.default_content unless enclosers.nil?
        end

      end # class PlatformObject
    end # module WatirWebDriver
  end # module Platforms
end # module SpecTest

Dir["#{File.dirname(__FILE__)}/../web_objects/**/*.rb"].each { |file| require file }

class String
  def is_integer?
    true if Integer(self) rescue false
  end
end